@@ -45,10 +45,14 @@ class Client:
         self, sources: List["Source"], forever: bool = True
     ):
         async def do_draw(s: "Source"):
+            canvas = await self.get_canvas()
+            await s.update_fix_queue(canvas)
             val = s.get_next_pixel()
             if not val:
                 return
             x, y, p = val
+            if canvas[x, y] == p:
+                return
             try:
                 await self.set_pixel(x, y, p)
                 return
